;**************************************************************************
;  RECEPTOR  RF PARA CONTROLE REMOTOS ON/OFF 8 CANAIS SIMULANDO HT658
;
;
;   RECEPTOR RF PARA CONTROLE REMOTOS ON/OFF 8 CANAIS
; 
;   PARA RECEPO UTILIZA UM PIC 16F628A
;   BLOG: 'LARIOS.TECNOLOGIA.WS'
;
;   CLAUDIO LRIOS        INICIO: 05-08-2012  TERMINO:05-08-2012
;   ATUALIZAO
;	DEIVID FLORIANI		  INICIO: 27-04-2013  TERMINO:27-04-2013
;**************************************************************************
;   CONTROLA AT 8 RELES DE SAIDA, COM OPERAO ON/OFF
;   TRABALHA EM CONJUNTO COM O TX DE CDIGO ASM: Trans
; 
;*************************************************************************** 

#INCLUDE <P16F628A.INC>	;ARQUIVO PADRO MICROCHIP PARA 16F628A
__CONFIG  _BODEN_ON & _CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _DATA_CP_OFF & _MCLRE_OFF & _INTRC_OSC_NOCLKOUT

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                    PAGINAO DE MEMRIA                         *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;DEFINIO DE COMANDOS DE USURIO PARA ALTERAO DA PGINA DE MEMRIA

#DEFINE	BANK0	BCF STATUS,RP0	;SETA BANK 0 DE MEMRIA
#DEFINE	BANK1	BSF STATUS,RP0	;SETA BANK 1 DE MAMRIA

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                         VARIVEIS                               *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DOS NOMES E ENDEREOS DE TODAS AS VARIVEIS UTILIZADAS 
; PELO SISTEMA

	CBLOCK	0x20	;ENDEREO INICIAL DA MEMRIA DE USURIO

	RES1
	RES0
	RES1A
	RES0A
	FLAGS
	BITCOUNTER
	LC
	HC
	CONT
	R0
	R1
	R2
	NOTX
	NOTX1
	AUX
	TEMPO1
	TEMPO2
	INFORMA

	ENDC			;FIM DO BLOCO DE MEMRIA		

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                        FLAGS INTERNOS                           *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS FLAGS UTILIZADOS PELO SISTEMA

#DEFINE		FLAG	FLAGS,0          ;FLAG DE SITUAO
#DEFINE		FLAG1	FLAGS,1     	 ;FLAG DE NOVA LEITURA

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                         CONSTANTES                              *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODAS AS CONSTANTES UTILIZADAS PELO SISTEMA


TMAX    EQU  .50       ;TEMPO MAXIMO DO SINAL LOW
QBYTE   EQU  .16        ;QUANTIDADE DE BYTES A SER RECEBIDO 
VNOTX1  EQU  .1        ;HIGH TEMPO MAXIMO SEM TRANSMISSO PARA ZERAR SAIDAS
VNOTX   EQU  .50     ;LOW TEMPO MAXIMO SEM TRANSMISSO PARA ZERAR SAIDAS

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                           ENTRADAS                              *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS PINOS QUE SERO UTILIZADOS COMO ENTRADA
; RECOMENDAMOS TAMBM COMENTAR O SIGNIFICADO DE SEUS ESTADOS (0 E 1)

#DEFINE 	CHAVE1		PORTA,6		  ;BOTO PARA ESCOLHA DE CDIGO


;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                           SADAS                                *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; DEFINIO DE TODOS OS PINOS QUE SERO UTILIZADOS COMO SADA
; RECOMENDAMOS TAMBM COMENTAR O SIGNIFICADO DE SEUS ESTADOS (0 E 1)

#DEFINE 	SIN 	PORTA,0  		 ;PINO 17  IN RF
#DEFINE		LED1	PORTA,1			 ;LED INDICA PRIMEIRO CDIGO	
#DEFINE		LED2	PORTA,2			 ;LED INDICA SEGUNDO  CDIGO	
#DEFINE		LED3	PORTA,3			 ;LED INDICA TERCEIRO CDIGO
				
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                       VETOR DE RESET                            *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

	ORG	0x00		;ENDEREO INICIAL DE PROCESSAMENTO
	GOTO	INICIO  ;DESVIA PARA O INCIO



	
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                    INCIO DA INTERRUPO                        *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; AS INTERRUPES NO SERO UTILIZADAS, POR ISSO PODEMOS SUBSTITUIR
; TODO O SISTEMA EXISTENTE NO ARQUIVO MODELO PELO APRESENTADO ABAIXO
; ESTE SISTEMA NO  OBRIGATRIO, MAS PODE EVITAR PROBLEMAS FUTUROS

	ORG	0x04		;ENDEREO INICIAL DA INTERRUPO
	RETFIE

;*********************************************************************************
;
;						TRATAMENTO DA INTERRUPO
;
;*********************************************************************************
	

;***********************************************************************
;		FIM DA INTERRUPO
;***********************************************************************
;RETORNA DA INTERRUPO

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     INICIO DO PROGRAMA                          *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	
INICIO
	CLRF	PORTA		;LIMPA O PORTA
	CLRF	PORTB		;LIMPA O PORTB

	BANK1

	MOVLW  	B'00001000' 	;DETERMINA A FREQNCIA DO OSCILADOR INTERNO PARA 4MHZ
	MOVWF   PCON

	MOVLW	B'01000001'
	MOVWF	TRISA		
						
	MOVLW	B'00000000'
	MOVWF	TRISB		
	MOVLW	B'00000000'
	MOVWF	OPTION_REG	
	MOVLW	B'00000000'
	MOVWF	INTCON		;TODAS AS INTERRUPES DESLIGADAS	

	BANK0				;RETORNA PARA O BANCO 0
	
	MOVLW	B'00000111'
	MOVWF	CMCON		;DEFINE O MODO DE OPERAO DO COMPARADOR ANALGICO
	CLRF   	FLAGS   ;APAGA REGISTRADOR DE FLAGS
    MOVLW  	VNOTX
    MOVWF  	NOTX
	MOVLW  	VNOTX1
    MOVWF  	NOTX1


;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     INICIALIZAO DAS VARIVEIS                 *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	CLRF	INFORMA
	MOVLW  	.170      ;CDIGO INICIAL DO RX
    MOVWF  	INFORMA
	BSF		LED1      ;LED REFERENTE AO CDIGO1
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                     ROTINA PRINCIPAL                            *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

MAIN 
  	 CLRF  	RES1
	 CLRF  	RES0
     MOVLW  QBYTE  ;PREPARA PARA A RECEPO COM NUMERO DE BITS A RECEBER
	 MOVWF  BITCOUNTER
 L0
     BCF   	FLAG   ;LIMPA FLAG E REGISTRADORES DE RECEPO
     CLRF 	HC
     CLRF  	LC
L1

L1H    
SS5:
	 BTFSS	CHAVE1
	 GOTO   $+2
	 GOTO	TESTE
     BTFSS 	FLAG     ;TESTA FLAG DE HUM/ZERO RECEBIDO
	 GOTO  	L2       ;FLAG=0
     BTFSS 	SIN      ;SIN ? TESTA SINAL DE ENTRADA DO RECEPTOR DE RF
	 GOTO   M0       ; SIN=0
L2
	 BTFSS 	SIN      ;SIN ?
     GOTO  	LA1      ;SIN=0
	 BSF  	FLAG     ;SIN=1
     INCF  	HC,F
     BTFSC 	STATUS,Z
     DECF  	HC,F
     GOTO  	LA2	   ;VAI CONTAR TEMPO DE 50 MICRO SEGUNDOS
LA1
    INCF  	LC,F     ;INCREMENTA REGISTRADOR DE LOW SIGNAL NA ENTRADA RF
    BTFSC 	STATUS,Z
    DECF  	LC,F
LA2
 	MOVLW  	.11  ;.11 = 50 MICROSEG    (ACERTA TEMPO ENTRE LEITURAS DA ENTRADA DE RF)
    MOVWF  	CONT
    DECFSZ 	CONT,F
    GOTO 	$-1

	DECFSZ 	NOTX,F
    GOTO  	SEG_RECP
    MOVLW  	VNOTX
    MOVWF  	NOTX
	DECFSZ  NOTX1,F
    GOTO   	SEG_RECP
	CLRF  	PORTB   ;DESLIGA TODAS AS SAIDAS, SE NO HOUVER SINAL RF
  
SEG_RECP

    GOTO L1
M0
    MOVFW  	LC		    ;TESTA SE NO PASSOU DO LIMITE MAXIMO DE TEMPO EM LOW NA ENTRADA DE RF
    SUBLW  	TMAX
    BTFSS  	STATUS,C
    GOTO   	MAIN       ;C=0 (-)  -  LC>TMAX (REJEITAR DADO - PAUSA INICIAL)
    MOVFW  	HC			;OBTEM O VALOR DE 'CARRY'(HUM OU ZERO)
    SUBWF  	LC,W
    RRF    	RES1,F       ;DESLOCA  O BIT NOS REGISTRADORES DE RESULTADO
    RRF    	RES0,F
    DECFSZ  BITCOUNTER,F  ;DECREMENTA O CONTADOR DE BITS A SER RECEBIDO
    GOTO   	L0             ;LE PROXIMO BIT
    BTFSC  	FLAG1        ;TESTA SE  A PRIMEIRA OU A SEGUNDA RECEPO
	GOTO   	PULA1
    MOVFW  	RES1		;SALVA A PRIMEIRA LEITURA PARA COMPARAO COM UMA SEGUNDA
    MOVWF  	RES1A    
    MOVFW  	RES0
    MOVWF  	RES0A	
    BSF   	FLAG1			;SETA FLAG DE PRIMEIRA RECEPO
    GOTO  	MAIN

PULA1
	BCF 	FLAG1          ;SEGUNDA RECEPO, COMPARA COM A PRIMEIRA
    MOVFW  	RES1
    XORWF  	RES1A,W
    BTFSS  	STATUS,Z
    GOTO   	ERROTX
	MOVFW  	RES0
    XORWF  	RES0A,W
    BTFSS  	STATUS,Z
    GOTO   	ERROTX	  	;ERRO DE RECEPO
    GOTO   	ACIONAR     ;OK - BOA RECEPO

ERROTX
	CLRF    RES1A     	; APAGA RESULTADOS DE COMPARAO
    CLRF    RES0A
    GOTO    MAIN

ACIONAR   
    MOVFW	INFORMA   	;CODIGO DO NOSSO TRANSMISSOR
    XORWF   RES0,W
    BTFSS   STATUS,Z
    GOTO    MAIN       	;NO  O NOSSO TRANSMISSOR, VOLTA AO INICIO
 	COMF   	RES1,W     	;COMPLEMENTA A SAIDA
    MOVWF   PORTB     	;PEM NA SAIDA 
	MOVLW  	VNOTX      	;RECARGA DO REGISTRADOR DE TX RECEBIDO
    MOVWF  	NOTX
	MOVLW  	VNOTX      	;RECARGA DO REGISTRADOR DE TX RECEBIDO
    MOVWF  	NOTX1
    GOTO 	MAIN

TESTE
	CALL 	DELAY_CHAVE
VERIFICA1
    BTFSS 	CHAVE1
	GOTO 	MAIN
	DECFSZ 	TEMPO1,F
	GOTO 	VERIFICA1
	DECFSZ 	TEMPO2,F
	GOTO 	VERIFICA1
CODIGO1
	BTFSS	LED1
	GOTO	CODIGO2
	BCF		LED1
	BSF		LED2
	CLRF	INFORMA
	MOVLW	.169	
	MOVWF	INFORMA
	GOTO	TRAVA
CODIGO2
	BTFSS	LED2
	GOTO	CODIGO3
	BCF		LED2
	BSF		LED3
	CLRF	INFORMA
	MOVLW	.168	
	MOVWF	INFORMA
	GOTO	TRAVA
CODIGO3
	BTFSS	LED3
	GOTO	MAIN
	BCF		LED3
	BSF		LED1
	CLRF	INFORMA
	MOVLW	.170	
	MOVWF	INFORMA
	GOTO	TRAVA
TRAVA
	BTFSS	CHAVE1
	GOTO	MAIN	
	GOTO	TRAVA

DELAY_CHAVE		  ;DELAY DE SEGURANA PARA EVITAR QUE UM RUDO ACIONE A TECLA = DELAY DE 80MS	
	CLRF 	TEMPO1
	CLRF 	TEMPO2
	MOVLW 	.50
	MOVWF 	TEMPO1
	MOVLW 	.100
	MOVWF 	TEMPO2
	RETURN

;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;*                       FIM DO PROGRAMA                           *
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	END			;OBRIGATRIO
